unit ConnectionModule;

interface

uses
  Aurelius.Drivers.Interfaces,
  Aurelius.Drivers.FireDac,
  FireDAC.Dapt, System.Generics.Collections,
  System.SysUtils, System.Classes, FireDAC.Stan.Intf, FireDAC.Stan.Option,
  FireDAC.Stan.Error, FireDAC.UI.Intf, FireDAC.Phys.Intf, FireDAC.Stan.Def,
  FireDAC.Stan.Pool, FireDAC.Stan.Async, FireDAC.Phys, FireDAC.ConsoleUI.Wait,
  Aurelius.Sql.MySQL, Aurelius.Schema.MySQL, Aurelius.Comp.Connection, Data.DB,
  FireDAC.Comp.Client, FireDAC.Phys.MySQL, FireDAC.Phys.MySQLDef, FireDAC.VCLUI.Wait,
  Aurelius.Mapping.Setup, Aurelius.Mapping.Attributes, FireDac.Comp.UI,
  Aurelius.Mapping.Explorer,
  XData.Comp.ConnectionPool, Aurelius.Engine.ObjectManager, Aurelius.Linq,   XData.Server.Module,
  XData.Service.Common,
  Aurelius.Comp.Manager;

type
  TFireDacMySqlConnection = class(TDataModule)
    Connection: TFDConnection;
    AureliusConnection1: TAureliusConnection;
    XDataConnectionPool1: TXDataConnectionPool;
    AureliusManager1: TAureliusManager;
  private

  public
    class function CreateConnection: IDBConnection;
    class function CreateFactory: IDBConnectionFactory;

    class function CreatePool(APoolSize: Integer): IDBConnectionPool;

    class function CreateEmptyQuery : TFDQuery;

    procedure CreateDynamicProps(ASetup: TMappingSetup);
    procedure DefineMappingSetup;
  end;

var
  FireDacMySqlConnection: TFireDacMySqlConnection;

implementation

{%CLASSGROUP 'System.Classes.TPersistent'}

uses
  XData.Aurelius.ConnectionPool,
  Aurelius.Drivers.Base,
  uTc_order, uTglobal_settings_user_fields;

{$R *.dfm}

{ TMyConnectionModule }

class function TFireDacMySqlConnection.CreateConnection: IDBConnection;
begin
  Result := FireDacMySqlConnection.AureliusConnection1.CreateConnection;
end;

class function TFireDacMySqlConnection.CreateEmptyQuery: TFDQuery;
begin
  var genQUery := TFDquery.Create(nil);
  Result := genQuery;
end;

class function TFireDacMySqlConnection.CreateFactory: IDBConnectionFactory;
begin
  Result := TDBConnectionFactory.Create(
    function: IDBConnection
    begin
      Result := CreateConnection;
    end
  );
end;

class function TFireDacMySqlConnection.CreatePool(APoolSize: Integer): IDBConnectionPool;
begin
  Result := TDBConnectionPool.Create(APoolSize, CreateFactory);
end;

procedure TFireDacMySqlConnection.CreateDynamicProps(ASetup: TMappingSetup);
var  PersonProps: TList<TDynamicProperty>;
begin
//exit;
    PersonProps := ASetup.DynamicProps[TC_order];
    PersonProps.Add(TDynamicProperty.Create('DynCon', 'GoodBye',
      TypeInfo(string),
      TDynamicColumn.Create('goodbye', [], 30)));

    var seznam := AureliusManager1.ObjManager.Find<Tglobal_Settings_user_fields>.Where(Linq['useInAurelius']=1).list;

    var aaaaa : Tglobal_Settings_user_fields;

    begin
       for aaaaa in seznam do
       begin
         if aaaaa.type_ = 'STRING' then
           PersonProps.Add(TDynamicProperty.Create('DynCon', aaaaa.name,
             TypeInfo(string),
               TDynamicColumn.Create(aaaaa.name, [], 100)));

         if aaaaa.type_ = 'INT' then
           PersonProps.Add(TDynamicProperty.Create('DynCon', aaaaa.name,
             TypeInfo(integer),
               TDynamicColumn.Create(aaaaa.name)));

         if aaaaa.type_ = 'DECIMAL' then
           PersonProps.Add(TDynamicProperty.Create('DynCon', aaaaa.name,
            TypeInfo(Double),
               TDynamicColumn.Create(aaaaa.name)));

         if aaaaa.type_ = 'DATE' then
           PersonProps.Add(TDynamicProperty.Create('DynCon', aaaaa.name,
            TypeInfo(TDate),
               TDynamicColumn.Create(aaaaa.name)));

         if aaaaa.type_ = 'DATETIME' then
           PersonProps.Add(TDynamicProperty.Create('DynCon', aaaaa.name,
            TypeInfo(TDatetime),
               TDynamicColumn.Create(aaaaa.name)));
       end;
    end;
end;

procedure TFireDacMySqlConnection.DefineMappingSetup;
var  MapSetup: TMappingSetup;
begin
  MapSetup := TMappingSetup.Create;
    try
       Self.CreateDynamicProps(MapSetup);
       TMappingExplorer.ReplaceDefaultInstance(TMappingExplorer.Create(MapSetup));
    finally
      MapSetup.Free;
    end;
end;

end.


