unit Server;

interface

uses
  ConnectionModule,
  XData.Server.Module,
  System.SysUtils,
  Sparkle.Middleware.Cors,
  Sparkle.Middleware.Compress,
  Sparkle.HttpSys.Server,
  Sparkle.HttpServer.Context,
  Sparkle.HttpServer.Module,
  XData.Openapi.service,
  XData.SwaggerUI.Service,
  xdata.Aurelius.Model,
  XData.Comp.Server, Sparkle.Comp.HttpSysDispatcher,
  Sparkle.HttpServer.Dispatcher,
  Sparkle.Middleware.Jwt,
  Sparkle.Middleware.Logging,
  Sparkle.Comp.JwtMiddleware,
  Xdata.Sys.Exceptions,
  XData.Aurelius.ModelBuilder,
  Aurelius.Mapping.Explorer;


procedure StartServer;
procedure StopServer;


type
  Tkrysa = class
  public
   procedure event(Sender: TObject;  Args: TEntityListArgs);
  end;

implementation

uses
  System.IOUtils, uconfig, udm.aurelius;

var
  SparkleServer: THttpSysServer;
  SparkleDispatcher : TSparkleHttpSysDispatcher;
  krysa : Tkrysa;

procedure Tkrysa.event(Sender: TObject;  Args: TEntityListArgs);
var
  User: IUserIdentity;
begin
  //nco
  User := TXDataOperationContext.Current.Request.User;
  if User = nil then
    raise EXDataHttpUnauthorized.Create('User not authenticated');
end;

procedure StartServer;
var
  Module, ModuleFunction : TXDataServerModule;
  mujServer : TXDataServer;
  mujMiddleWare: TSparkleJwtMiddleware;
  mujEvent : TEntityListEvent;

  Builder: TXDataModelBuilder;
  Model: TXDataAureliusModel;
  Explorer: TMappingExplorer;

begin
  if Assigned(SparkleServer) then
     Exit;

  Config.Load;

  krysa := Tkrysa.Create;

  ConnectionModule.FireDacMySqlConnection.Connection.Params.Database := Config.eMistrDatabase;
  ConnectionModule.FireDacMySqlConnection.Connection.Params.AddPair('Port', Config.eMistrPort.ToString);
  ConnectionModule.FireDacMySqlConnection.Connection.Params.AddPair('Server', Config.eMistrServer);
  try
    ConnectionModule.FireDacMySqlConnection.Connection.Connected := True;
  except
    exit;
  end;

  SparkleServer := THttpSysServer.Create;

  Explorer := TMappingExplorer.DefaultInstance;
     Model := TXDataAureliusModel.Create(Explorer);
       try
          Builder := TXDataModelBuilder.Create(Model);
             try
               Builder.UseOriginalClassNames := true;
               Builder.UseOriginalFieldNames := true;
               Builder.Build;
             finally
               Builder.Free;
             end;
       except
         Model.Free; // destroy the model in case an error occurs
         raise;
       end;

  ConnectionModule.FireDacMySqlConnection.DefineMappingSetup;

  Module := TXDataServerModule.Create(
    'http://+:8090/Tables',
    TFireDacMySqlConnection.CreatePool(20),Model
  );

  //if not assigned(dmAurelius) then dmAurelius := TdmAurelius.Create(nil);

//  ConnectionModule.FireDacMySqlConnection.DefineMappingSetup;

  // Uncomment line below to enable CORS in the server
  //Module.AddMiddleware(TCorsMiddleware.Create);

  // Uncomment line below to allow compressed responses from server
  //Module.AddMiddleware(TCompressMiddleware.Create);

  Module.AddMiddleware(TCorsMiddleware.Create);
  Module.AccessControlAllowOrigin := '*';

  SparkleServer.AddModule(Module);

//  ModuleFunction := TXDataServerModule.Create(
//    'http://+:8090/emistr',
//    TFireDacMySqlConnection.CreatePool(20),TXDataAureliusModel.Get('eMISTR'));
//
//  // Uncomment line below to enable CORS in the server
//  //ModuleFunction.AddMiddleware(TCorsMiddleware.Create);
//
//  // Uncomment line below to allow compressed responses from server
//  //ModuleFunction.AddMiddleware(TCompressMiddleware.Create);
//
//  ModuleFunction.AddMiddleware(TCorsMiddleware.Create);
//  ModuleFunction.AccessControlAllowOrigin := '*';
//  ModuleFunction.AddMiddleware(TAnonymousMiddleware.Create(nil));
//
//  SparkleServer.AddModule(ModuleFunction);

  mujServer := TXDataServer.Create(nil);
  mujServer.Model.Get('eMISTR');
  mujServer.BaseUrl := 'http://+:8090/Functions';
  mujServer.SwaggerOptions.Enabled := True;
  mujServer.SwaggerUIOptions.Enabled := True;

  mujServer.Pool := (connectionmodule.FireDacMySqlConnection.XDataConnectionPool1);

  SparkleDispatcher := TSparkleHttpSysDispatcher.Create(nil);
  mujServer.Dispatcher := SparkleDispatcher;

  mujMiddleWare := TSparkleJwtMiddleware.Create(nil);
  mujMiddleWare.Secret := '123456789';

  mujServer.MiddlewareList.Add(mujMiddleWare);

  //mujevent := krysa.event;

  //mujServer.OnEntityList := mujEvent; //procedure(Sender: TObject;  Args: TEntityListArgs)   begin   end;

  //TXDataModelBuilder.LoadXmlDoc(XDataServer.Model);
  mujServer.Model.Title := 'eMISTR API';
  mujServer.Model.Version := '1.0';
  mujServer.Model.Description :=
  '![eMISTR](https://emistr.cz/wp-content/uploads/2020/03/logo.svg)'#13#10+
    '### Overview'#13#10 +
    'This is an API for **eMISTR** operations. '#13#10 +
    'Feel free to browse and execute several operations like list, add, update, remove data'#13#10#13#10 +
    '### More info'#13#10 +
    'Build with [TMS XData](https://www.tmssoftware.com/site/xdata.asp), from [TMS Software](https://www.tmssoftware.com).' +
    'A Delphi framework for building REST servers'#13#10 +
    '[![TMS Software](https://download.tmssoftware.com/business/tms-logo-small.png)](https://www.tmssoftware.com)';


  SparkleDispatcher.Start;



//  RegisterOpenAPIService;
//  RegisterSwaggerUIService;

  SparkleServer.Start;
end;

procedure StopServer;
begin
  FreeAndNil(SparkleServer);
end;

initialization
  SparkleServer := nil;
finalization
  StopServer;
  SparkleDispatcher.Stop;
  FreeAndNil(SparkleDispatcher);
end.
